"use strict";
const tslib_1 = require("tslib");
const event_target_proxy_1 = tslib_1.__importDefault(require("event-target-proxy"));
const error_classes_1 = require("./error-classes");
const from_evented_stream_1 = tslib_1.__importDefault(require("./from-evented-stream"));
function fromChildProcess(cp) {
    const { stdout, stderr } = cp;
    const proxifiedStdOut = event_target_proxy_1.default(stdout, ({ event, listener }) => event === 'error'
        ? ((error) => listener(new error_classes_1.StdOutError(stdout, error)))
        : listener);
    const proxifiedStdErr = event_target_proxy_1.default(stderr, ({ event, listener }) => event === 'error'
        ? ((error) => listener(new error_classes_1.StdErrError(stderr, error)))
        : listener);
    const createEvtMod = (modfn) => (event, fn) => {
        switch (event) {
            case 'close':
                modfn(cp, 'close', fn);
                break;
            case 'error':
                modfn(cp, 'error', fn);
                modfn(proxifiedStdOut, 'error', fn);
                modfn(proxifiedStdErr, 'error', fn);
                break;
            case 'data':
                modfn(proxifiedStdOut, 'data', fn);
                modfn(proxifiedStdErr, 'data', fn);
                break;
        }
    };
    const addListener = createEvtMod((target, event, fn) => target.addListener(event, fn));
    const removeListener = createEvtMod((target, event, fn) => target.removeListener(event, fn));
    return from_evented_stream_1.default({ addListener, removeListener });
}
module.exports = fromChildProcess;
//# sourceMappingURL=from-child-process.js.map