/// <reference types="node" />
import { EventedStream } from 'evented-stream-types';
import StringWritable from '../utils/string-writable';
import { ArrayLike, MaybeFunc } from '../utils/types';
export { EventedStream };
export declare type Code = number;
export declare type Sequence = ArrayLike<Code>;
export declare type Data = AsyncIterable<Code> | Iterable<Code>;
export declare type IterableStream = AsyncIterable<Buffer | string>;
export declare type SequenceFunc = MaybeFunc<Sequence, [SequenceFunc.Param]>;
export declare namespace SequenceFunc {
    interface Param {
        readonly splitter: Splitter;
        readonly currentLine: Sequence;
        readonly leadingCharacters: ReadonlyArray<Sequence>;
    }
}
export interface Splitter {
    readonly data: Data;
    readonly prefix: SequenceFunc;
    readonly suffix: SequenceFunc;
}
export interface Element {
    readonly format: ReadonlyArray<Sequence>;
    readonly reset: Sequence;
    readonly main: Sequence;
    readonly prefix: Sequence;
    readonly suffix: Sequence;
}
export interface Writable {
    write(buffer: Buffer): void;
}
export interface ChildProcess<Chunk, Err = Error> {
    readonly stdout: EventedStream<Chunk, Err>;
    readonly stderr: EventedStream<Chunk, Err>;
    readonly addListener: ChildProcess.ListenerModifier;
    readonly removeListener: ChildProcess.ListenerModifier;
}
export declare namespace ChildProcess {
    interface ListenerModifier {
        (event: 'close', listener: CloseEventListener): void;
        (event: 'error', listener: ErrorEventListener): void;
    }
    type CloseEventListener = () => void;
    type ErrorEventListener = () => void;
}
export interface ConstructorOptions {
    readonly data: Data;
    readonly prefix?: SequenceFunc;
    readonly suffix?: SequenceFunc;
}
export interface ToStringOptions extends StringWritable.ConstructorOptions {
    readonly finalNewLine?: boolean;
}
